"""
METAR static values
"""

#: Strings signifying the start of the remarks section of a METAR
METAR_RMK = [
    " BLU",
    " BLU+",
    " WHT",
    " GRN",
    " YLO",
    " AMB",
    " RED",
    " ALL",
    " BECMG",
    " TEMPO",
    " INTER",
    " NOSIG",
    " RMK",
    " WIND",
    " QFE",
    " QFF",
    " INFO",
    " RWY",
    " CHECK",
]
